//=============================================================================
// ReversedCTFGame.
// $Id: ReversedCTFGame.uc,v 1.3 2002/10/17 12:33:07 Andreas Exp $
//=============================================================================
class ReversedCTFGame extends xCTFGame config;

function PreBeginPlay()
{
  local CTFBase thisFlagBase;
  
  Super.PreBeginPlay();
  
  ForEach DynamicActors(class'CTFBase', thisFlagBase)
    if ( thisFlagBase.DefenderTeamIndex == 0 )
      thisFlagBase.FlagType = class'ReversedRedFlag';
    else
      thisFlagBase.FlagType = class'ReversedBlueFlag';
}

function SetTeamFlags()
{
  local CTFFlag F;

  // associate flags with teams
  ForEach AllActors(Class'CTFFlag',F)
  {
    F.Team = Teams[F.TeamNum];
    F.Team.HomeBase = F.HomeBase;
    CTFTeamAI(F.Team.AI).EnemyFlag = F;
    if ( F.TeamNum == 0 )
      CTFTeamAI(Teams[1].AI).FriendlyFlag = F;
    else
      CTFTeamAI(Teams[0].AI).FriendlyFlag = F;
  }
}

function ScoreFlag(Controller Scorer, CTFFlag theFlag)
{
  local float Dist,oppDist;
  local int i;
  local float ppp,numtouch;
  local vector FlagLoc;

  if ( Scorer.PlayerReplicationInfo.Team != theFlag.Team ) {  // was "== theFlag.Team"
    Scorer.AwardAdrenaline(ADR_Return);
    FlagLoc = TheFlag.Position().Location;
    Dist = vsize(FlagLoc - TheFlag.HomeBase.Location);
    
    if ( TheFlag.TeamNum != 0 )  // was "== 0"
      oppDist = vsize(FlagLoc - Teams[1].HomeBase.Location);
    else
        oppDist = vsize(FlagLoc - Teams[0].HomeBase.Location); 
  
    if (GameStats!=None)
      GameStats.GameEvent("flag_returned",""$theFlag.Team.TeamIndex,Scorer.PlayerReplicationInfo);

    BroadcastLocalizedMessage(class'CTFMessage', 1, Scorer.PlayerReplicationInfo, None, TheFlag.Team);
    
    if ( Dist > 1024 ) {
      // figure out who's closer
        
      if ( Dist <= oppDist ) {  // In your team's zone
        Scorer.PlayerReplicationInfo.Score += 3;
        
        if ( GameStats != None )
          GameStats.ScoreEvent(Scorer.PlayerReplicationInfo,3,"flag_ret_friendly");
      }
      else {
        Scorer.PlayerReplicationInfo.Score += 5;
        
        if ( GameStats != None )
          GameStats.ScoreEvent(Scorer.PlayerReplicationInfo,5,"flag_ret_enemy");
        
        if ( oppDist <= 1024) {  // Denial
          Scorer.PlayerReplicationInfo.Score += 7;
          if ( GameStats != None )
            GameStats.ScoreEvent(Scorer.PlayerReplicationInfo,7,"flag_denial");
        }
          
      }          
    } 
    return;
  }
  
  // Figure out Team based scoring.
  if ( TheFlag.FirstTouch != None ) {  // Original Player to Touch it gets 5
    if ( GameStats != None )
      GameStats.ScoreEvent(TheFlag.FirstTouch.PlayerReplicationInfo,5,"flag_cap_1st_touch");
    
    TheFlag.FirstTouch.PlayerReplicationInfo.Score += 5;
  }
  
  // Guy who caps gets 5
  Scorer.PlayerReplicationInfo.Score += 5;
  IncrementGoalsScored(Scorer.PlayerReplicationInfo);
    Scorer.AwardAdrenaline(ADR_Goal);
  
  // Each player gets 20/x but it's guarenteed to be at least 1 point but no more than 5 points 
  numtouch = 0;  
  for(i = 0; i < TheFlag.Assists.length; i++) {
    if ( TheFlag.Assists[i] != None )
      numtouch = numtouch + 1.0;
  }
  
  ppp = FClamp(20 / numtouch, 1, 5);
  
  for (i = 0; i < TheFlag.Assists.length; i++) {
    if ( TheFlag.Assists[i] != None ) {
      if ( GameStats!=None )
        GameStats.ScoreEvent(TheFlag.Assists[i].PlayerReplicationInfo,ppp,"flag_cap_assist");
      
      TheFlag.Assists[i].PlayerReplicationInfo.Score += int(ppp);
    }
  }
  
  // Apply the team score
  Scorer.PlayerReplicationInfo.Team.Score += 1.0;
  if ( GameStats != None ) {
    GameStats.ScoreEvent(Scorer.PlayerReplicationInfo,5,"flag_cap_final");
    GameStats.TeamScoreEvent(Scorer.PlayerReplicationInfo.Team.TeamIndex,1,"flag_cap");  
    GameStats.GameEvent("flag_captured",""$theflag.Team.TeamIndex,Scorer.PlayerReplicationInfo);
  }  
  BroadcastLocalizedMessage( class'CTFMessage', 0, Scorer.PlayerReplicationInfo, None, TheFlag.Team );
  AnnounceScore(Scorer.PlayerReplicationInfo.Team.TeamIndex);
  CheckScore(Scorer.PlayerReplicationInfo);
  
    if ( bOverTime )
    EndGame(Scorer.PlayerReplicationInfo,"timelimit");
}

defaultproperties
{
     GameName="Reversed Capture the Flag"
     HUDType="ReversedCTF.HudReversedCTF"
     //TeamAIType(0)=class'ReversedCTFTeamAI'
     //TeamAIType(1)=class'ReversedCTFTeamAI'
}
